package com.ruoyi.project.VRdemo.service.impl;

import com.ruoyi.common.exception.ServiceException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * 导入模板方法
 */
public abstract class BulkImportService<T> {

    // 单次导入批次最大数
    private static final int BATCH_COUNT = 500;

    // 单次导入批次列表
    private final List<T> batchDataList = new ArrayList<>();

    // 导入数据列表
    private  List<T> toBeImportedList;

    private int successNum = 0;    // 成功消息数
    // 成功消息详情
    private final StringBuilder successMsg = new StringBuilder();

    private int failureNum = 0;    // 失败消息数
    // 失败消息详情
    private final StringBuilder failureMsg = new StringBuilder();

    private static final Logger logger = LoggerFactory.getLogger(BulkImportService.class);


    /**
     * 导入模板方法
     *
     * @param dataList      数据列表
     * @param updateSupport 是否更新已存在的用户数据
     */
    @Transactional(isolation = Isolation.READ_COMMITTED)
    public String importData(List<T> dataList, boolean updateSupport) {
        //导入list校验
        validateData(dataList);

        toBeImportedList = dataList;

        for (T item : dataList) {
            try {
                // 进行导入操作
                processItem(item, updateSupport);
            } catch (Exception e) {
                // 输出异常信息和上下文信息到日志
                logger.error("导入数据时发生异常：", e);
                logger.error("异常发生时的数据：{}", item); // 这里的 item 是导致异常的数据对象
                // 处理导入失败的情况
                handleImportFailure(item);
            }
        }

        return returnMessage();
    }

    // 校验列表方法
    protected void validateData(List<T> dataList) {
        if (dataList == null || dataList.isEmpty()) {
            throw new ServiceException("导入列表数据不能为空！");
        }
    }

    // 校验具体数据方法-不重写即不校验
    protected boolean isInvalid(T item) {
        if (item == null) {
            throw new ServiceException("导入数据不能为空！");
        }
        return true;
    }

    // 校验不通过操作
    protected void handleInvalid(T item) {
        handleImportFailure(item);
        logger.error("导入数据时发生异常：{}", getInvalidMessage(item));
        logger.error("校验不通过的数据：{}", item);
    }

    // 校验不通过返回消息方法
    protected String getInvalidMessage(T item) {
        return getItemTitle(item) + "校验不通过";
    }

    /**
     * 校验数据，并插入数据库
     */
    protected void processItem(T item, Boolean updateSupport) {
        if (!isInvalid(item)) {
            handleInvalid(item); // 处理校验不通过的情况
            return;
        }

        Long i = 0L;
        if (updateSupport) {
            i = checkExistingItem(item);
        }
        if (i != 0L) {
            handleUpdate(item, i); // 更新数据
        } else {
            batchDataList.add(item); // 导入列表插入数据

            if (batchDataList.size() >= BATCH_COUNT || isLastItem(item)) {
                handleInsert(); // 进行数据导入
                batchDataList.clear();
            }
        }
    }

    // 更新数据操作
    protected void handleUpdate(T item, Long id) {
        if (updateItem(item, id)) {
            successNum++;
            successMsg.append("成功更新数据：").append(getItemTitle(item)).append("\n");
        } else {
            failureNum++;
            successMsg.append("更新数据失败：").append(getItemTitle(item)).append("\n");
        }
    }

    // 插入数据操作
    protected void handleInsert() {
        boolean result = executeBatch(batchDataList); // 列表数据导入数据库
        for (T t : batchDataList) {
            if (result) {
                handleImportSuccess(t);
            } else {
                handleImportFailure(t);
            }
        }
    }

    // 判断是否为最后一个数据方法
    protected boolean isLastItem(T item) {
        return toBeImportedList.indexOf(item) == toBeImportedList.size() - 1;
    }

    /**
     * 校验数据是否已存在方法-进行查找数据操作
     */
    protected abstract Long checkExistingItem(T item);

    /**
     * 更新数据方法-进行更新数据操作
     */
    protected abstract boolean updateItem(T item, Long id);

    /**
     * 批量插入数据库方法-进行批量插入数据库操作
     */
    protected abstract boolean executeBatch(List<T> batchDataList);


    protected void handleImportSuccess(T item) {
        successMsg.append("导入成功，").append(getItemTitle(item)).append("\n");
        successNum++;
    }

    protected void handleImportFailure(T item) {
        failureMsg.append("导入失败，").append(getItemTitle(item)).append("\n");
        failureNum++;
    }

    /**
     * 获取具体数据标题方法
     */
    protected abstract String getItemTitle(T item);

    /**
     * 返回消息
     */
    protected String returnMessage() {
        if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确或插入数据库出错，错误如下：\n");
            throw new ServiceException(failureMsg.toString());
        } else {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：\n");
        }

        return successMsg.toString();
    }

}
